package com.example.demo.controller;

import com.example.demo.model.Todo;
import com.example.demo.service.TodoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/api")
public class TodoReactiveController {

    @Autowired
    TodoService service;

    // http://localhost:9030/api/todos
    @GetMapping("/todos")
    @ResponseStatus(HttpStatus.OK)
    public Flux<Todo> getAllTodos() {
        return service.findAll();
    }

    // http://localhost:9030/api/todos/completed?completed=true
    @GetMapping("/todos/completed")
    @ResponseStatus(HttpStatus.OK)
    public Flux<Todo> findByCompleted(@RequestParam(name = "completed", defaultValue = "false") boolean completed) {
        return service.findByCompletedFlag(completed);
    }

    // http://localhost:9030/api/todos/pageable?page=0&size=10
    @GetMapping("/todos/pageable")
    @ResponseStatus(HttpStatus.OK)
    public Mono<Page<Todo>> findAllBy(@RequestParam(name = "page", defaultValue = "0") int page,
                                      @RequestParam(name = "size", defaultValue = "10") int size) {
        /*
        The given code initializes a Pageable object in Java using the PageRequest.of(page, size) method. The Pageable interface is used for managing paging and pagination, allowing you to specify parameters like page number and items per page. PageRequest is a concrete implementation of Pageable, and of(page, size) is a static factory method within PageRequest used to create a Pageable object with specific page and size values. This Pageable object is typically employed in data retrieval tasks, enabling developers to request a particular page of data from a larger dataset, commonly in the context of database queries or API operations.
         */
        Pageable pageable = PageRequest.of(page, size);
        return service.findAllBy(pageable);
    }
}
