package com.example.demo.repository;

import com.example.demo.model.Todo;
import org.springframework.data.domain.Pageable;
import org.springframework.data.mongodb.repository.ReactiveMongoRepository;
import org.springframework.stereotype.Repository;
import reactor.core.publisher.Flux;

/*
A ReactiveMongoRepository is an interface provided by the Spring Data MongoDB project for building reactive data access operations in a Spring Boot application. It extends the ReactiveCrudRepository interface and specializes in MongoDB data storage. Developers can define custom repository interfaces by extending ReactiveMongoRepository and defining method signatures for various data operations like saving, finding, updating, and deleting documents. These methods are automatically implemented by Spring Data MongoDB, providing a straightforward way to interact with a MongoDB database using reactive programming concepts, such as Mono and Flux. This repository interface simplifies the development of reactive data access layers in Spring Boot applications, promoting non-blocking and efficient handling of database interactions in reactive web applications and microservices.
 */
@Repository
public interface TodoRepository extends ReactiveMongoRepository<Todo, Integer> {

    Flux<Todo> findByCompleted(boolean isCompleted);

    /*
    The method findAllBy(Pageable pageable) is a part of a reactive programming paradigm, specifically associated with a Flux. It is used to retrieve a paginated list of Todo objects. The Pageable parameter allows you to specify the pagination settings, such as the desired page number, the number of items per page, sorting criteria, and more. The method returns a Flux of Todo objects, meaning it asynchronously emits these objects in a reactive stream, enabling efficient and non-blocking retrieval of paginated data. This is commonly used in scenarios where you want to retrieve a subset of Todo records, such as when implementing paginated APIs or database queries in reactive applications.
     */
    Flux<Todo> findAllBy(Pageable pageable);
}
